//==============================================
//      File rPlugins.pas
//
//      Components library for Delphi and BCB++.
//        .
//      Plugins support.
//
//      Copyright 2001-02 Polaris Software
//      http://polesoft.da.ru
//      mailto: PolarisLib@mail.ru
//==============================================
unit rPlugins;

{$I POLARIS.INC}

interface

uses
  Windows, Classes, Registry, SysUtils, rUtils, rVCLUtils, rConst,
  rDLLloader, {$IFDEF POLARIS_D6}RxStrUtils{$ELSE}StrUtils{$ENDIF};

//=============================================================================

type

  TrPlugin = class(TrDLLloader)
  private
    FPlugName: String;
    FPlugType: String;
    FDefault: Boolean;
    FOptParams: TStringList;
    procedure UpdateInfo;
    procedure SetDefault(Value: Boolean);
    function CheckRequiredSoft: Boolean;
    function GetOptParams: String;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    function Execute(const Params1, Params2, Params3: String): String;
    property PlugName: String read FPlugName;
    property PlugType: String read FPlugType;
    property OptParams: String read GetOptParams;
    property IsDefault: Boolean read FDefault;
  end;

  TrPlugins = class(TComponent)
  private
    FPlugins: TList;
    FRegKey: String;
    FPlugsPath: String;
    FFileMask: String;
    FRequiredVersion: String;
    Reg: TRegistry;
    function GetPlugin(Number: Integer): TrPlugin;
    procedure SetPlugsPath(const Value: String);
    function CheckVersionFormat: Boolean;
    procedure SetRequiredVersion(const Value: String);
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveParamsToReg;
    property Plugins[Index: Integer]: TrPlugin read GetPlugin;
    function ExecutePlugin(
      const PlugName: String;
      const PlugType: String;
      const Param1, Param2, Param3: String): String;
    function GetPlugTypes: String;
    function GetPlugNames(const PlugType: String): String;
    function GetDefaultPlug(const PlugType: String): String;
    function GetPlugByOptParams(PlugType, ParamName, ParamValue: String): String;
    function GetExistsPlug(
      const PlugType, PlugName: String;
      DefaultIfNotExists: Boolean {$IFDEF POLARIS_D4}= False{$ENDIF}
      ): String;
    function GetPluginByName(const PlugName: String): TrPlugin;
    function PlugExists(const PlugType, PlugName: String): Boolean;
    procedure SetDefaultPlug(const PlugType, PlugName: String);
    property PlugsPath: String read FPlugsPath write SetPlugsPath;
  published
    property RegKey: String read FRegKey write FRegKey;
    property FileMask: String read FFileMask write FFileMask stored True;
    property RequiredVersion: String read FRequiredVersion write SetRequiredVersion
      stored CheckVersionFormat;
  end;

implementation

const
  cFileMask = '*.dll';
  cExecuteProc = 'Execute';
  cGetInfoProc = 'GetInfo';

//=============================================================================
//
// = TrPlugin =
//
//=============================================================================

constructor TrPlugin.Create(aOwner: TComponent);
begin
  inherited;
  FOptParams := TStringList.Create;
  FOptParams.Sorted := True;
  FOptParams.Duplicates := dupIgnore;
  FDefault := False;
end;

destructor TrPlugin.Destroy;
begin
  FOptParams.Free;
  inherited;
end;

procedure TrPlugin.SetDefault(Value: Boolean);
begin
  FDefault := Value;
end;

function TrPlugin.CheckRequiredSoft: Boolean;
begin
  if not Active
    then Result := False
    else Result := Assigned(EntryPoint[cExecuteProc]) and
                   Assigned(EntryPoint[cGetInfoProc]);
end;

function TrPlugin.Execute(const Params1, Params2, Params3: String): String;
type
  TExecuteProc = function(const Params1, Params2, Params3: pChar): pChar; stdcall;
var
  vNotActive: Boolean;
begin
  vNotActive := not Active;
  if vNotActive then Open;
  Result := TExecuteProc(EntryPoint[cExecuteProc])
    (pChar(Params1), pChar(Params2), pChar(Params3));
  if vNotActive then Close;
end;

procedure TrPlugin.UpdateInfo;
type
  TInfoProc = procedure(
    var sFullName: pChar;
    var sPlugType: pChar;
    var sOptParams: pChar); stdcall;
var
  p1, p2, p3: pChar;
begin
  TInfoProc(EntryPoint[cGetInfoProc])(p1, p2, p3);
  FPlugName := p1;
  FPlugType := p2;
  FOptParams.Text := p3;
end;

function TrPlugin.GetOptParams: String;
begin
  Result := FOptParams.Text;
end;

//=============================================================================
//
// = TrPlugins =
//
//=============================================================================
constructor TrPlugins.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Reg := TRegistry.Create;
  FPlugins := TList.Create;
  FFileMask := cFileMask;
end;

//=============================================================================
destructor TrPlugins.Destroy;
begin
  SaveParamsToReg;
  FPlugins.Free;
  Reg.Free;
  inherited Destroy;
end;

//=============================================================================
//    
function TrPlugins.GetPlugin(Number: Integer): TrPlugin;
begin
  Result := TrPlugin(FPlugins[Number]);
end;

//=============================================================================
//    
function TrPlugins.GetPluginByName(const PlugName: String): TrPlugin;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to FPlugins.Count - 1 do
    if Plugins[i].PlugName = PlugName then begin
      Result := TrPlugin(FPlugins[i]);
      break;
    end;
end;

//=============================================================================
//  
function TrPlugins.ExecutePlugin(
  const PlugName: String;
  const PlugType: String;
  const Param1, Param2, Param3 : String): String;
var
  i : Integer;
begin
  Result := Format(srPluginNotFound, [PlugName, PlugType]);
  for i := 0 to FPlugins.Count - 1 do
    if ( (PlugName = Plugins[i].PlugName) and
         ( (PlugType = '') or (PlugType = Plugins[i].PlugType) )
       )
       or
       ( (PlugType = Plugins[i].PlugType) and (PlugName = '') and
         (Plugins[i].IsDefault)
       )
    then begin
      Result := Plugins[i].Execute(Param1, Param2, Param3);
      break;
    end;
end;

//=============================================================================
//   
procedure TrPlugins.SetPlugsPath(const Value: String);
var
  i: Integer;
  sr: TSearchRec;
  sl: TStringList;
  r: HKEY;
  s, sError: String;
  plug: TrPlugin;
  vbOpenError: Boolean;
begin

  plug := nil;
  sl := TStringList.Create;
  try

    for i := FPlugins.Count - 1 downto 0 do begin
      Plugins[i].Free;
      FPlugins.Delete(i);
    end;

    FPlugsPath := Value;
    if FindFirst(AddSlash(FPlugsPath) + FFileMask, faAnyFile, sr) = 0 then begin
      sl.Add(sr.Name);
      while FindNext(sr) = 0 do sl.Add(sr.Name);
    end;
    FindClose(sr);

    sError := EmptyStr;
    for i := 0 to sl.Count - 1 do begin
      if plug = nil then begin
        plug := TrPlugin.Create(Self);
        if FRequiredVersion <> '' then plug.RequiredVersion := FRequiredVersion;
      end;
      plug.LibraryName := AddSlash(FPlugsPath) + sl.Strings[i];
      vbOpenError := False;
      try
        plug.Open;
      except
        on E: Exception do begin
          sError := sError + crlf + E.Message;
          vbOpenError := True;
        end;
      end;
      if not vbOpenError and plug.CheckRequiredSoft then begin
        plug.UpdateInfo;
        plug.Close;
        if not PlugExists('', plug.PlugName) then FPlugins.Add(plug);
        plug := nil;
      end;
      if plug <> nil then plug.Close;
    end;
    if plug <> nil then begin
      plug.Free;
      plug := nil;
    end;

    sl.Text := GetPlugTypes;
    for i := 0 to sl.Count - 1 do
      SetDefaultPlug(sl.Strings[i],
        ExtractWord(1, GetPlugNames(sl.Strings[i]), [#13]));
    if FRegKey <> '' then begin
      SeparateRegKey(FRegKey, r, s);
      Reg.RootKey := r;
      Reg.OpenKey(s + '\Plugins\Default', True);
      for i := 0 to sl.Count - 1 do
        if Reg.ValueExists(sl.Strings[i])
          then SetDefaultPlug(sl.Strings[i], Reg.ReadString(sl.Strings[i]));
    end;

    if sError <> EmptyStr then raise Exception.Create(sError);

  finally
    Reg.CloseKey;
    sl.Free;
    if plug <> nil then plug.Free;
  end;

end;

//=============================================================================
//   
function TrPlugins.GetPlugTypes: String;
var
  i  : Integer;
  sl : TStringList;
begin
  sl := TStringList.Create;
  sl.Sorted := True;
  sl.Duplicates := dupIgnore;
  for i := 0 to FPlugins.Count - 1 do sl.Add(Plugins[i].PlugType);
  Result := sl.Text;
  sl.Free;
end;

//=============================================================================
//  
function TrPlugins.GetPlugNames(const PlugType: String): String;
var
  i  : Integer;
  sl : TStringList;
begin
  sl := TStringList.Create;
  sl.Sorted := True;
  sl.Duplicates := dupIgnore;
  for i := 0 to FPlugins.Count - 1 do
    if (PlugType = '') or (PlugType = Plugins[i].PlugType)
      then sl.Add(Plugins[i].PlugName);
  Result := sl.Text;
  sl.Free;
end;

//=============================================================================
//    
procedure TrPlugins.SetDefaultPlug(const PlugType, PlugName: String);
var
  i : Integer;
begin
  if PlugExists(PlugType, PlugName) and (PlugName <> '') then
    for i := 0 to FPlugins.Count - 1 do
      if Plugins[i].PlugType = PlugType then
        Plugins[i].SetDefault(Plugins[i].PlugName = PlugName);
end;

//=============================================================================
//    
function TrPlugins.GetDefaultPlug(const PlugType: String): String;
var
  i : Integer;
begin
  Result := '';
  for i := 0 to FPlugins.Count - 1 do
    if (Plugins[i].PlugType = PlugType) and (Plugins[i].IsDefault) then begin
      Result := Plugins[i].PlugName;
      Break;
    end;
end;

//=============================================================================
//   
function TrPlugins.PlugExists(const PlugType, PlugName: String): Boolean;
var
  r : Integer;
  sl  : TStringList;
begin
  sl := TStringList.Create;
  sl.Sorted := True;
  sl.Text := GetPlugNames(PlugType);
  if sl.Count > 0 then begin
    if PlugName = '' then Result := True
                     else Result := sl.Find(PlugName, r);
  end
  else Result := False;
  sl.Free;
end;

//=============================================================================
//      
function TrPlugins.GetPlugByOptParams(PlugType, ParamName, ParamValue: String
  ): String;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to FPlugins.Count - 1 do
    if ((PlugType = '') or (Plugins[i].PlugType = PlugType)) and
       (Plugins[i].FOptParams.Values[ParamName] = ParamValue)
    then begin
      Result := Plugins[i].PlugName;
      Break;
    end;
end;

//=============================================================================
//    ( )     ( )
function TrPlugins.GetExistsPlug(
  const PlugType, PlugName: String;
  DefaultIfNotExists: Boolean
  ): String;
begin
  Result := '';
  if (PlugType <> '') and DefaultIfNotExists and
    ( (PlugName = '') or not PlugExists('', PlugName) )
    then Result := GetDefaultPlug(PlugType)
  else
  if (PlugName <> '') and PlugExists(PlugType, PlugName) then Result := PlugName;
end;

//=============================================================================
//   
function TrPlugins.CheckVersionFormat: Boolean;
var
  MS, LS: DWORD;
begin
  Result := SeparateVersion(FRequiredVersion, MS, LS);
end;

//=============================================================================
//      
procedure TrPlugins.SetRequiredVersion(const Value: String);
var
  MS, LS: DWORD;
begin
  if SeparateVersion(Value, MS, LS) then FRequiredVersion := Value;
end;

//=============================================================================
//  
procedure TrPlugins.SaveParamsToReg;
var
  sl: TStringList;
  i: Integer;
  s: String;
  r: HKEY;
begin
  if FRegKey <> '' then begin
    sl := TStringList.Create;
    try
      sl.Text := GetPlugTypes;
      SeparateRegKey(FRegKey, r, s);
      Reg.RootKey := r;
      Reg.OpenKey(s + '\Plugins\Default', True);
      for i := 0 to sl.Count - 1 do
        Reg.WriteString(sl.Strings[i], GetDefaultPlug(sl.Strings[i]));
    finally
      Reg.CloseKey;
      sl.Free;
    end;
  end;
end;

//=============================================================================
end.

